#!/usr/bin/perl
#
# Launcher for UML2SVG using XALAN
# Author: Stephane GALLAND <galland@arakhne.org>
# Copyright (C) 2007, Stephane GALLAND, under GPL
#
# Creating for the Debian package of UML2SVG.

use strict;

use Getopt::Long;
use Pod::Usage ;
use File::Basename ;
use File::Spec ;

#---------------- FOUND SCRIPT LOCATION ON HOST
my $VERSION = "0.18";
my $PERLSCRIPTDIR ;
my $PERLSCRIPTNAME ;
my $PERLCOMMONSCRIPTNAME ;
BEGIN{
  # Where is this script?
  $PERLSCRIPTDIR = "$0";
  my $scriptdir = dirname( $PERLSCRIPTDIR );
  while ( -e $PERLSCRIPTDIR && -l $PERLSCRIPTDIR ) {
    $PERLSCRIPTDIR = readlink($PERLSCRIPTDIR);
    if ( substr( $PERLSCRIPTDIR, 0, 1 ) eq '.' ) {
      $PERLSCRIPTDIR = File::Spec->catfile( $scriptdir, "$PERLSCRIPTDIR" ) ;
    }
    $scriptdir = dirname( $PERLSCRIPTDIR );
  }
  $PERLSCRIPTNAME = basename( $PERLSCRIPTDIR ) ;
  if ($PERLSCRIPTNAME =~ /^([^\-]+)/) {
    $PERLCOMMONSCRIPTNAME = "$1";
  }
  else {
    $PERLCOMMONSCRIPTNAME = $PERLSCRIPTNAME;
  }
  $PERLSCRIPTDIR = dirname( $PERLSCRIPTDIR ) ;
  $PERLSCRIPTDIR = File::Spec->rel2abs( "$PERLSCRIPTDIR" );
  # Push the path where the script is to retreive the arakhne.org packages
  push(@INC,"$PERLSCRIPTDIR");

}



#---------------- USAGE AND HELP FUNCTIONS
sub show_usage($$$) {

	show_manual(@_);

  my $exitval = shift;
  my $scriptdir = shift;
  my $scriptname = shift;

  my $basename = "$scriptname";
  $basename =~ s/\.[^.]*$//;

  my $sharedir = "/usr/share";

  my @searchdirs = (
	File::Spec->catdir("$scriptdir",'pod'),
	File::Spec->catdir("$scriptdir",'man'),
	File::Spec->catdir("$scriptdir",File::Spec->updir(),'pod'),
	File::Spec->catdir("$scriptdir",File::Spec->updir(),'man'),
	File::Spec->catdir("$sharedir",'doc',"$basename"),
	File::Spec->catdir("$sharedir",'doc',"$basename"),
	File::Spec->catdir("$sharedir",'doc',"$basename",'pod'),
	File::Spec->catdir("$sharedir",'doc',"$basename",'man'),
	);

  my @langs = ();

  if (($ENV{'LANG'})&&($ENV{'LANG'} =~ /^([^_.\-]+)/)) {
    push @langs, "$1";
  }

  push @langs, 'en';

  foreach my $lang (@langs) {
    foreach my $dir (@searchdirs) {
      my $pod = File::Spec->catdir("$dir","${basename}_${lang}.pod");
      if ( -r "$pod" ) {
        print "$pod\n";
        pod2usage(-exitval => $exitval, -input => "$pod");
        exit $exitval;
      }
    }
  }

  print STDERR ("unable to find the documentation file for $basename\n");
  exit $exitval;
}

sub show_manual($$$) {
  my $exitval = shift;
  my $scriptdir = shift;
  my $scriptname = shift;

  my $basename = "$scriptname";
  $basename =~ s/\.[^.]*$//;

  my $sharedir = "/usr/share";

  my @searchdirs = (
	File::Spec->catdir("$scriptdir",'pod'),
	File::Spec->catdir("$scriptdir",'man'),
	File::Spec->catdir("$scriptdir",File::Spec->updir(),'pod'),
	File::Spec->catdir("$scriptdir",File::Spec->updir(),'man'),
	File::Spec->catdir("$sharedir",'doc',"$basename"),
	File::Spec->catdir("$sharedir",'doc',"$basename"),
	File::Spec->catdir("$sharedir",'doc',"$basename",'pod'),
	File::Spec->catdir("$sharedir",'doc',"$basename",'man'),
	);

  my @langs = ();

  if (($ENV{'LANG'})&&($ENV{'LANG'} =~ /^([^_.\-]+)/)) {
    push @langs, "$1";
  }

  push @langs, 'en';

  foreach my $lang (@langs) {
    foreach my $dir (@searchdirs) {
      my $pod = File::Spec->catdir("$dir","${basename}_${lang}.pod");
      if ( -r "$pod" ) {
        print "$pod\n";
        use Pod::Perldoc;
        @ARGV = ( "$pod" );
        Pod::Perldoc->run();
        exit $exitval;
      }
    }
  }

  print STDERR ("unable to find the documentation file for $basename\n");
  exit $exitval;
}

#---------------- VERSION FUNCTION
sub show_version($$$) {
  my $exitval = shift;
  my $scriptdir = shift;
  my $scriptname = shift;

  print STDOUT "uml2svg $VERSION\n";

  exit $exitval;
}


#---------------- MAIN PROGRAM

my %options = ();

# Get command line options
if (!GetOptions( "help" => \$options{'help'},
		 "output=s" => \$options{'output'},
		 "umlversion=s" => \$options{'uml-version'},
		 "diagramname=s" => \$options{'diagram-name'},
		 "diagramnumber=s" => \$options{'diagram-number'},
		 "diagramid=s" => \$options{'diagram-id'},
		 "style=s" => \$options{'style'},
		 "xalan=s" => \$options{'xalan'},
		 "version" => \$options{'version'},
                 )) {
  show_usage(1,"$PERLSCRIPTDIR","$PERLCOMMONSCRIPTNAME");
}

# Does user want to show version?
if ($options{'version'}) {
  show_version(3,"$PERLSCRIPTDIR","$PERLCOMMONSCRIPTNAME");
}

# Does user want to show help?
if ($options{'help'}) {
  show_manual(2,"$PERLSCRIPTDIR","$PERLCOMMONSCRIPTNAME");
}

# Find the XSLT style
my $xsltstyle = $options{'style'};
if ((!$xsltstyle)||(!(-f "$xsltstyle"))) {
	$xsltstyle = File::Spec->catfile("$PERLSCRIPTDIR","uml2svg.xsl");
	if (!(-f "$xsltstyle")) {
	  print STDERR "unable to find the file uml2svg.xsl required for the conversion\n";
	  show_usage(4,"$PERLSCRIPTDIR","$PERLCOMMONSCRIPTNAME");
	}
}

# Check the input filename
if ((@ARGV)&&($ARGV[0])&&(!(-f $ARGV[0]))&&
    ($ARGV[0] !~ /\.xmi$/i)) {
	$ARGV[0] .= ".xmi";
}
if ((!@ARGV)||(!$ARGV[0])||(!(-f $ARGV[0]))) {
	print STDERR "no input file specified or not existing file\n";
	show_usage(3,"$PERLSCRIPTDIR","$PERLCOMMONSCRIPTNAME");
}

# Check the output filename
if (!$options{'output'}) {
	$options{'output'} = $ARGV[0];
	$options{'output'} =~ s/\.xmi$/\.svg/i;
	$options{'output'} =~ s/\.uml$/\.svg/i;
	$options{'output'} =~ s/\.xml$/\.svg/i;
	$options{'output'} .= '.svg'
		unless ($options{'output'} =~ /\.svg$/i);
}

# Treat parameters
my @parameters = ();

if ($options{'uml-version'}) {
	push @parameters, '-param',
		'UmlStdVersion', $options{'uml-version'};
}

if ($options{'diagram-name'}) {
	push @parameters, '-param',
		'SelectedDiagram', $options{'diagram-name'};
}

if ($options{'diagram-number'}) {
	push @parameters, '-param',
		'SelectedDiagramNumber', $options{'diagram-number'};
}

if ($options{'diagram-id'}) {
	push @parameters, '-param',
		'SelectedDiagramId', $options{'diagram-id'};
}

if (!$options{'xalan'}) {
  $options{'xalan'} = File::Spec->catfile('','usr','share','java','xalan2.jar');
}
if (!(-f $options{'xalan'})) {
  print STDERR "unable to find the XALAN library:\n".$options{'xalan'}."\n";
  show_usage(6,"$PERLSCRIPTDIR","$PERLCOMMONSCRIPTNAME");
}

# Build the shell command
my $classpath = join(':',
			$options{'xalan'},
			'');
my @shell_command = (
	'java',
        '-cp', $classpath,
	'org.apache.xalan.xslt.Process',
	'-OUT', $options{'output'},
	@parameters,
	'-XSL', "$xsltstyle",
	'-IN', $ARGV[0]
	);

#use Data::Dumper;
#die(Dumper(\@shell_command));

exec @shell_command;

die("unable to run the XSLT processor");
__END__
